
#define RT_FORMAT_DTA_104   0x000001
#define RT_FORMAT_DTA_105   0x000002
#define RT_FORMAT_DTA_108   0x000004
#define RT_FORMAT_DTA_110   0x000008
#define RT_FORMAT_DTA_111   0x000010
#define RT_FORMAT_DTA_114   0x000020
#define RT_FORMAT_DTA_117   0x000040
#define RT_FORMAT_DTA_118   0x000080
#define RT_FORMAT_DTA_119   0x000100

#define RT_FORMAT_DTA_105_AND_OLDER   (RT_FORMAT_DTA_105 | RT_FORMAT_DTA_104)
#define RT_FORMAT_DTA_108_AND_OLDER   (RT_FORMAT_DTA_108 | RT_FORMAT_DTA_105_AND_OLDER)
#define RT_FORMAT_DTA_110_AND_OLDER   (RT_FORMAT_DTA_110 | RT_FORMAT_DTA_108_AND_OLDER)
#define RT_FORMAT_DTA_111_AND_OLDER   (RT_FORMAT_DTA_111 | RT_FORMAT_DTA_110_AND_OLDER)
#define RT_FORMAT_DTA_114_AND_OLDER   (RT_FORMAT_DTA_114 | RT_FORMAT_DTA_111_AND_OLDER)
#define RT_FORMAT_DTA_117_AND_OLDER   (RT_FORMAT_DTA_117 | RT_FORMAT_DTA_114_AND_OLDER)
#define RT_FORMAT_DTA_118_AND_OLDER   (RT_FORMAT_DTA_118 | RT_FORMAT_DTA_117_AND_OLDER)

#define RT_FORMAT_DTA_118_AND_NEWER   (RT_FORMAT_DTA_118 | RT_FORMAT_DTA_119)
#define RT_FORMAT_DTA_117_AND_NEWER   (RT_FORMAT_DTA_117 | RT_FORMAT_DTA_118_AND_NEWER)
#define RT_FORMAT_DTA_114_AND_NEWER   (RT_FORMAT_DTA_114 | RT_FORMAT_DTA_117_AND_NEWER)
#define RT_FORMAT_DTA_111_AND_NEWER   (RT_FORMAT_DTA_111 | RT_FORMAT_DTA_114_AND_NEWER)
#define RT_FORMAT_DTA_110_AND_NEWER   (RT_FORMAT_DTA_110 | RT_FORMAT_DTA_111_AND_NEWER)
#define RT_FORMAT_DTA_108_AND_NEWER   (RT_FORMAT_DTA_108 | RT_FORMAT_DTA_110_AND_NEWER)
#define RT_FORMAT_DTA_105_AND_NEWER   (RT_FORMAT_DTA_105 | RT_FORMAT_DTA_108_AND_NEWER)

#define RT_FORMAT_DTA                 (RT_FORMAT_DTA_104 | RT_FORMAT_DTA_105_AND_NEWER)

#define RT_FORMAT_SAV_COMP_NONE 0x001000
#define RT_FORMAT_SAV_COMP_ROWS 0x002000
#define RT_FORMAT_SAV_COMP_ZLIB 0x004000
#define RT_FORMAT_POR           0x008000

#define RT_FORMAT_SAV_COMP  (RT_FORMAT_SAV_COMP_ROWS | RT_FORMAT_SAV_COMP_ZLIB)
#define RT_FORMAT_SAV       (RT_FORMAT_SAV_COMP_NONE | RT_FORMAT_SAV_COMP)

#define RT_FORMAT_SPSS      (RT_FORMAT_SAV | RT_FORMAT_POR)

#define RT_FORMAT_SAS7BDAT_32BIT_COMP_NONE    0x010000
#define RT_FORMAT_SAS7BDAT_32BIT_COMP_ROWS    0x020000
#define RT_FORMAT_SAS7BDAT_32BIT (RT_FORMAT_SAS7BDAT_32BIT_COMP_NONE | RT_FORMAT_SAS7BDAT_32BIT_COMP_ROWS)

#define RT_FORMAT_SAS7BDAT_64BIT_COMP_NONE    0x040000
#define RT_FORMAT_SAS7BDAT_64BIT_COMP_ROWS    0x080000
#define RT_FORMAT_SAS7BDAT_64BIT (RT_FORMAT_SAS7BDAT_64BIT_COMP_NONE | RT_FORMAT_SAS7BDAT_64BIT_COMP_ROWS)

#define RT_FORMAT_SAS7BDAT_COMP_NONE (RT_FORMAT_SAS7BDAT_32BIT_COMP_NONE | RT_FORMAT_SAS7BDAT_64BIT_COMP_NONE)
#define RT_FORMAT_SAS7BDAT_COMP_ROWS (RT_FORMAT_SAS7BDAT_32BIT_COMP_ROWS | RT_FORMAT_SAS7BDAT_64BIT_COMP_ROWS)

#define RT_FORMAT_SAS7BDAT  (RT_FORMAT_SAS7BDAT_32BIT | RT_FORMAT_SAS7BDAT_64BIT)

#define RT_FORMAT_SAS7BCAT 0x100000

#define RT_FORMAT_XPORT_5  0x200000
#define RT_FORMAT_XPORT_8  0x400000

#define RT_FORMAT_XPORT (RT_FORMAT_XPORT_5 | RT_FORMAT_XPORT_8)

#define RT_FORMAT_SAS   (RT_FORMAT_SAS7BDAT | RT_FORMAT_XPORT)

#define RT_FORMAT_ALL       (RT_FORMAT_DTA | RT_FORMAT_SPSS | RT_FORMAT_SAS)
